--[[
	MeshInstance: Instancing the positions of mesh points on vector layers
	1. Use NAME for the name of the source layer, and NAME.dup for all of the other layers.
	2. Set this script for all of the layers (ie. for the original and for the instances)
	3. Modify always the source layer only.
]]--

-- written by Fazek

function LayerScript(moho)
	local mesh= moho:Mesh()
	if (mesh == nil) then return end

	local ownLayer= moho.layer
	local name= ownLayer:Name()
	local tab
	local mi= ownLayer.FA_MeshInst
	local len= mesh:CountPoints()

	if (string.sub(name,-4) == ".dup") then
--copy instance
		if (mi and ownLayer.FA_MeshInstName == name) then
	--cached
			if (mi[2] ~= moho.frame) then
				print("MeshInstance: wrong order - swap the source and the copy!")
				return
			end
			tab= mi[1]

			if (mi[3] < len) then len= mi[3] end

			for p= 1, len do
				local pt= mesh:Point(p - 1)
				pt.fPos.x= tab[p][1]
				pt.fPos.y= tab[p][2]
			end
		else
			local srcName= string.sub(name,1,-5)

		-- recursive search for the source

			local stack= {}
			local sp= 0
			local document= moho.document

			for i= 0, document:CountLayers() - 1 do
				local layer= document:Layer(i)
				local parent= nil
				local sub= 0

				while true do

					if (layer:IsGroupType()) then
						table.insert(stack, { parent, sub - 1 })
						sp= sp + 1
						parent= moho:LayerAsGroup(layer)
						sub= parent:CountLayers()
					elseif (layer:Name() == srcName) then
						local mi= layer.FA_MeshInst

						if (mi) then

							if (mi[2] == moho.frame) then
								tab= mi[1]

								if (mi[3] < len) then len= mi[3] end

								for p= 1, len do
									local pt= mesh:Point(p - 1)
									pt.fPos.x= tab[p][1]
									pt.fPos.y= tab[p][2]
								end
								ownLayer.FA_MeshInst= mi
								ownLayer.FA_MeshInstName= name
								return
							end
						end
						print("MeshInstance: wrong order - swap the source and the copy!")
						return
					end

					if (sub > 0) then
						sub= sub - 1
						layer= parent:Layer(sub)
					else
						sub= -1

						while (sp > 0) do
							parent, sub= stack[sp][1], stack[sp][2]
							table.remove(stack)
							sp= sp - 1

							if (sub >= 0) then
								layer= parent:Layer(sub)
								break
							end
						end

						if (sub < 0) then
							break
						end
					end
				end
			end
		end
	else
--source
		tab= {}

		for p= 0, len - 1 do
			local pt= mesh:Point(p)
			table.insert(tab, { pt.fPos.x, pt.fPos.y })
		end
	--trick: FA_MeshInst won't change in the address space (no orphans)

		if (mi and not ownLayer.FA_MeshInstName == name) then
	--cached
			mi[1]= tab
			mi[2]= moho.frame
			mi[3]= len
		else
			ownLayer.FA_MeshInst= { tab, moho.frame, len }
			ownLayer.FA_MeshInstName= name

			mi= ownLayer.FA_MeshInst
			name= name..".dup"

	--avoid orphaning: update all dependencies

		-- recursive search for the copy

			local stack= {}
			local sp= 0
			local document= moho.document

			for i= 0, document:CountLayers() - 1 do
				local layer= document:Layer(i)
				local parent= nil
				local sub= 0

				while true do

					if (layer:IsGroupType()) then
						table.insert(stack, { parent, sub - 1 })
						sp= sp + 1
						parent= moho:LayerAsGroup(layer)
						sub= parent:CountLayers()
					elseif (layer:LayerType() == MOHO.LT_VECTOR and layer:Name() == name) then
						layer.FA_MeshInst= mi
						layer.FA_MeshInstName= name
					end

					if (sub > 0) then
						sub= sub - 1
						layer= parent:Layer(sub)
					else
						sub= -1

						while (sp > 0) do
							parent, sub= stack[sp][1], stack[sp][2]
							table.remove(stack)
							sp= sp - 1

							if (sub >= 0) then
								layer= parent:Layer(sub)
								break
							end
						end

						if (sub < 0) then
							break
						end
					end
				end
			end
		end
	end
end
